package app;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * PropertyManager class to manage property handling
 */
public class PropertyManager {
    private Properties properties = new Properties();

    /**
     * Initialize the properties by reading them
     * @throws IOException
     */
    public void readProperties() throws IOException {
        BufferedInputStream stream = new BufferedInputStream(new FileInputStream("options.properties"));
        properties.load(stream);
        stream.close();
    }

    /**
     * Get a property by given key
     * @param key used to get a value
     * @return property by key
     */
    public String getProperty(String key) {
        return properties.getProperty(key);
    }

    /**
     * Change property by given key to given value
     * @param key which key you want to edit
     * @param val to change given value of key
     * @throws IllegalArgumentException if key doesn't exist
     */
    public void changeProperty(String key, String val) throws IllegalArgumentException {
        if (!properties.containsKey(key)) throw new IllegalArgumentException();
        properties.setProperty(key, val);
    }

}